

use "$data\GLD_All_last.dta", clear

* Focus on those working

keep if lstatus==1




* Replace missing isco versions

replace isco_version="isco_2008" if isco_version=="" & countrycode=="ARM"  
replace isco_version="isco_1988" if isco_version=="" & countrycode=="CHL" & year==1990
replace isco_version="isco_1988" if isco_version=="" & countrycode=="TUR" & year==2000
replace isco_version="isco_2008" if isco_version=="" & countrycode=="ZWE" & year==2019
replace isco_version="isco_1988" if isco_version=="" & countrycode=="LKA" & year>=1992 & year<=2012
replace isco_version="isco_1968" if isco_version=="" & countrycode=="IDN" & year==2014



 
** Recode everything to ISCO 2008 version
iscogen job08_1 = isco08(occup_isco) if isco_version=="isco_1968", from(isco68)
iscogen job08   = isco08(occup_isco) if   isco_version=="isco_1988", from(isco88)

gen occup_isco_08=occup_isco 			if isco_version=="isco_2008"
destring occup_isco_08, replace

replace occup_isco_08=job08 			if job08!=.
replace occup_isco_08=job08_1 			if job08_1!=.

*** This now adds the information on the two and one digit occupation level from colombia and indonesia 
gen occup2=occup_orig*100				if (countrycode=="COL" & year>=2007 & year<=2021) 
gen occup3=occup_orig*1000				if (countrycode=="IDN" & year>=2016 & year<=2019)
replace occup3=occup_orig*1000			if (countrycode=="TUN")


iscogen occup_isco_08_superficial = isco08(occup2) 		if countrycode=="COL" & year>=2007 & year<=2021, from(isco68)
replace occup_isco_08_superficial=occup3				if countrycode=="IDN" & year>=2016 & year<=2019
replace occup_isco_08_superficial=occup3				if countrycode=="TUN"

replace occup_isco_08_superficial=occup_isco_08 if occup_isco_08_superficial==.

* This adds the labels, if needed
iscolbl isco08  occup_isco_08
iscolbl isco08  occup_isco_08_superficial


drop if occup_isco_08_superficial==. 


** Taking the 3 digit average code for the last missings
replace occup_isco_08=1110 if occup_isco_08==1111
replace occup_isco_08=2650 if occup_isco_08==2659
replace occup_isco_08=3410 if occup_isco_08==3413
replace occup_isco_08=4210 if occup_isco_08==4213
replace occup_isco_08=5160 if occup_isco_08==5161
replace occup_isco_08=6220 if occup_isco_08==6226
replace occup_isco_08=7110 if occup_isco_08==7133
replace occup_isco_08=8150 if occup_isco_08==8155
replace occup_isco_08=8150 if occup_isco_08==8159
replace occup_isco_08=9330 if occup_isco_08==9332
replace occup_isco_08=9500 if occup_isco_08==9510
replace occup_isco_08=9610 if occup_isco_08==9613


drop if occup_isco_08_superficial==0


*** After an exchange with GLD focal person we drop Armenia and (later) Colombia for quality reasons

*drop if countrycode=="COL"
drop if countrycode=="ARM"

*** Add the AIOE information and append USA

merge m:1 occup_isco_08 using "$data\SOCISAIOE.dta"   

keep if _merge==3

drop SOC_1 SOCTitle  SOC2010   _merge

save "$data\GLD_All_AI_last.dta", replace



**** Add USA


use 	"$data\I2D2\USA_GLD.dta", clear
rename occup_orig CensusCode1
tostring CensusCode1, replace

gen 	CensusCode=string(real(CensusCode1), "%004.0f") 
replace CensusCode="" if CensusCode=="-001" 
drop if CensusCode=="."
**** Merge the SOC with AIOE after transforming the occupation variable

merge m:1 CensusCode using "$data\CensusAIOE.dta"
keep if _merge==3
drop _merge
save "$data\I2D2\USA_AIOE.dta", replace

use "$data\GLD_All_AI_last.dta", replace

append using "$data\I2D2\USA_AIOE.dta", force


compress
save "$data\GLD_All_AI_last.dta", replace


